@@ -1,23 +1,17 @@
-#################################################################################
+## Copyright (C) 2012 Marius Hofert, Ivan Kojadinovic, Martin Maechler, and Jun Yan
 ##
-##   R package Copula by Jun Yan and Ivan Kojadinovic Copyright (C) 2009
+## This program is free software; you can redistribute it and/or modify it under
+## the terms of the GNU General Public License as published by the Free Software
+## Foundation; either version 3 of the License, or (at your option) any later
+## version.
 ##
-##   This file is part of the R package copula.
+## This program is distributed in the hope that it will be useful, but WITHOUT
+## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+## FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+## details.
 ##
-##   The R package copula is free software: you can redistribute it and/or modify
-##   it under the terms of the GNU General Public License as published by
-##   the Free Software Foundation, either version 3 of the License, or
-##   (at your option) any later version.
-##
-##   The R package copula is distributed in the hope that it will be useful,
-##   but WITHOUT ANY WARRANTY; without even the implied warranty of
-##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-##   GNU General Public License for more details.
-##
-##   You should have received a copy of the GNU General Public License
-##   along with the R package copula. If not, see <http://www.gnu.org/licenses/>.
-##
-#################################################################################
+## You should have received a copy of the GNU General Public License along with
+## this program; if not, see <http://www.gnu.org/licenses/>.
 
 
 strictify <- function(val,status)
@@ -30,25 +24,24 @@ strictify <- function(val,status)
 debye1 <- function(x, give=FALSE, strict=TRUE){
   attr <- attributes(x)
   x.vec <- as.vector(x)
-  jj <- .C("debye_1",
+  jj <- .C(debye_1_C,
            as.double(abs(x.vec)),  ## added abs by JY
            as.integer(length(x.vec)),
            val=as.double(x.vec),
            err=as.double(x.vec),
-           status=as.integer(0*x.vec),
-           PACKAGE="copula"
-           )
+           status=as.integer(0*x.vec))
+
   val <- ifelse(x.vec >=0, jj$val, jj$val - x.vec / 2) ## k = 1, Frees & Valdez 1998, p.9
   err <- jj$err
   status <- jj$status
   attributes(val) <- attr
-  attributes(err) <- attr  
+  attributes(err) <- attr
   attributes(status) <- attr
 
   if(strict){
     val <- strictify(val,status)
   }
-  
+
   if(give){
       return(list(val=val,err=err,status=status))
   } else {
@@ -59,55 +52,51 @@ debye1 <- function(x, give=FALSE, strict=TRUE){
 debye2 <- function(x, give=FALSE, strict=TRUE){
   attr <- attributes(x)
   x.vec <- as.vector(x)
-  jj <- .C("debye_2",
+  jj <- .C(debye_2,
            as.double(abs(x.vec)),  ## added abs by JY
            as.integer(length(x.vec)),
            val=as.double(x.vec),
            err=as.double(x.vec),
-           status=as.integer(0*x.vec),
-           PACKAGE="copula"
-           )
+           status=as.integer(0*x.vec))
   val <- ifelse(x.vec >= 0, jj$val, jj$val - x.vec * 2/ 3) ## k = 2
   err <- jj$err
   status <- jj$status
   attributes(val) <- attr
-  attributes(err) <- attr  
+  attributes(err) <- attr
   attributes(status) <- attr
 
 
   if(strict){
     val <- strictify(val,status)
   }
-  
+
   if(give){
       return(list(val=val,err=err,status=status))
   } else {
     return(val)
   }
-}  
+}
 
 ## debye3 <- function(x, give=FALSE, strict=TRUE){
 ##   attr <- attributes(x)
 ##   x.vec <- as.vector(x)
-##   jj <- .C("debye_3",
+##   jj <- .C(debye_3,
 ##            as.double(x.vec),
 ##            as.integer(length(x.vec)),
 ##            val=as.double(x.vec),
 ##            err=as.double(x.vec),
-##            status=as.integer(0*x.vec),
-##            PACKAGE="copula"
-##            )
+##            status=as.integer(0*x.vec))
 ##   val <- jj$val
 ##   err <- jj$err
 ##   status <- jj$status
 ##   attributes(val) <- attr
-##   attributes(err) <- attr  
+##   attributes(err) <- attr
 ##   attributes(status) <- attr
 
 ##   if(strict){
 ##     val <- strictify(val,status)
 ##   }
-  
+
 ##   if(give){
 ##       return(list(val=val,err=err,status=status))
 ##   } else {
@@ -130,20 +119,19 @@ debye2 <- function(x, give=FALSE, strict=TRUE){
 ##   err <- jj$err
 ##   status <- jj$status
 ##   attributes(val) <- attr
-##   attributes(err) <- attr  
+##   attributes(err) <- attr
 ##   attributes(status) <- attr
 
 ##   if(strict){
 ##     val <- strictify(val,status)
 ##   }
-  
+
 ##   if(give){
 ##       return(list(val=val,err=err,status=status))
 ##   } else {
 ##     return(val)
 ##   }
-## }  
-
+## }
 
 ## ## debye function is used for compute the assoc measure of frankCopula
 ## debye <- function(x, k, ...) {
