@@ -1,28 +1,20 @@
-#################################################################################
+## Copyright (C) 2012 Marius Hofert, Ivan Kojadinovic, Martin Maechler, and Jun Yan
 ##
-##   R package Copula by Jun Yan and Ivan Kojadinovic Copyright (C) 2009
+## This program is free software; you can redistribute it and/or modify it under
+## the terms of the GNU General Public License as published by the Free Software
+## Foundation; either version 3 of the License, or (at your option) any later
+## version.
 ##
-##   This file is part of the R package copula.
+## This program is distributed in the hope that it will be useful, but WITHOUT
+## ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+## FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+## details.
 ##
-##   The R package copula is free software: you can redistribute it and/or modify
-##   it under the terms of the GNU General Public License as published by
-##   the Free Software Foundation, either version 3 of the License, or
-##   (at your option) any later version.
-##
-##   The R package copula is distributed in the hope that it will be useful,
-##   but WITHOUT ANY WARRANTY; without even the implied warranty of
-##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-##   GNU General Public License for more details.
-##
-##   You should have received a copy of the GNU General Public License
-##   along with the R package copula. If not, see <http://www.gnu.org/licenses/>.
-##
-#################################################################################
+## You should have received a copy of the GNU General Public License along with
+## this program; if not, see <http://www.gnu.org/licenses/>.
 
 
-################################################################
-## Class fitCopula
-################################################################
+### Class fitCopula ############################################################
 
 setClass("fitCopula",
          representation(estimate = "numeric",
@@ -30,19 +22,18 @@ setClass("fitCopula",
                         method = "character",
                         loglik = "numeric",
                         convergence = "integer",
+                        optimOpts = "list",
                         nsample = "integer",
-                        copula = "copula"),
-         validity = function(object) TRUE,
-         contains = list()
+                        copula = "copula")
+         ## FIXME validity = function(object) TRUE
          )
 
 setClass("summaryFitCopula",
          representation(method = "character",
                         loglik = "numeric",
                         convergence = "integer",
-                        parameters = "data.frame"),
-         validity = function(object) TRUE,
-         contains = list()
+                        parameters = "data.frame")
+         ## FIXME validity = function(object) TRUE
          )
 
 showFitCopula <- function(object) {
@@ -51,6 +42,7 @@ showFitCopula <- function(object) {
   print(foo@parameters)
   if (!is.na(foo@loglik)) cat("The maximized loglikelihood is ", foo@loglik, "\n")
   if (!is.na(foo@convergence)) cat("The convergence code is ", foo@convergence, "\n")
+  invisible(object)
 }
 
 summaryFitCopula <- function(object) {
@@ -62,46 +54,46 @@ summaryFitCopula <- function(object) {
   dimnames(parameters) <-
     list(object@copula@param.names,
          c("Estimate", "Std. Error", "z value", "Pr(>|z|)"))
-  ret <- new("summaryFitCopula",
-             method = object@method,
-             loglik = object@loglik,
-             convergence = object@convergence,
-             parameters = parameters)
-  ret
+  ## MM{FIXME}: S4 class is overkill here
+  new("summaryFitCopula",
+      method = object@method,
+      loglik = object@loglik,
+      convergence = object@convergence,
+      parameters = parameters)
 }
 
 setMethod("show", signature("fitCopula"), showFitCopula)
 setMethod("summary", signature("fitCopula"), summaryFitCopula)
 
-################################################################
-## Wrapper function
-################################################################
+
+## Wrapper function ############################################################
 
 fitCopula <- function(copula, data,
-                      method = "mpl", 
+                      method = "mpl",
                       start = NULL,
                       lower = NULL, upper = NULL,
                       optim.control = list(maxit=1000),
                       optim.method = "BFGS",
                       estimate.variance = TRUE,
-                      hideWarnings = TRUE) {
-  if (method == "ml") 
-    fit <- fitCopula.ml(data, copula, start, lower, upper, optim.control,
-                        optim.method, estimate.variance, hideWarnings)
-  else if (method == "mpl")
-    fit <- fitCopula.mpl(copula, data, start, lower, upper, optim.control,
-                         optim.method, estimate.variance, hideWarnings)
-  else if (method == "itau")
-    fit <- fitCopula.itau(copula, data, estimate.variance)
-  else if (method == "irho")
-    fit <- fitCopula.irho(copula, data, estimate.variance)
-  else stop("Implemented methods are: ml, mpl, itau, and irho.")
-  fit
+                      hideWarnings = TRUE)
+{
+  switch(method,
+	 "ml" =
+	 fitCopula.ml(data, copula, start, lower, upper, optim.control,
+		      optim.method, estimate.variance, hideWarnings),
+	 "mpl" =
+	 fitCopula.mpl(copula, data, start, lower, upper, optim.control,
+		       optim.method, estimate.variance, hideWarnings),
+	 "itau" =
+	 fitCopula.itau(copula, data, estimate.variance),
+	 "irho" =
+	 fitCopula.irho(copula, data, estimate.variance),
+	 ## otherwise
+	 stop("Implemented methods are: ml, mpl, itau, and irho."))
 }
 
-###############################################################
-## fitCopula with maximizing pseudo-likelihood
-###############################################################
+
+## fitCopula with maximizing pseudo-likelihood #################################
 
 fitCopula.mpl <- function(copula, data, start=NULL,
                           lower=NULL, upper=NULL,
@@ -118,24 +110,24 @@ fitCopula.mpl <- function(copula, data, start=NULL,
     }
     else start <- copula@parameters
   }
-  
+
   fit <- fitCopula.ml(data, copula, start, lower, upper,
-                      optim.control, optim.method, FALSE, hideWarnings)
+                      optim.control, optim.method,
+                      estimate.variance=FALSE, hideWarnings)
   var.est <- if(estimate.variance) varPL(fit@copula, data) / nrow(data) else matrix(NA, q, q)
-  ans <- new("fitCopula",
-             estimate = fit@estimate,
-             var.est = var.est,
-             method = "maximum pseudo-likelihood",
-             loglik = fit@loglik,
-             convergence = fit@convergence,
-             nsample = nrow(data),
-             copula = fit@copula)
-  ans
+  new("fitCopula",
+      estimate = fit@estimate,
+      var.est = var.est,
+      method = "maximum pseudo-likelihood",
+      loglik = fit@loglik,
+      convergence = fit@convergence,
+      optimOpts = fit@optimOpts,
+      nsample = nrow(data),
+      copula = fit@copula)
 }
 
-###############################################################
-## fitCopula using inversion of Kendall's tau
-###############################################################
+
+## fitCopula using inversion of Kendall's tau ##################################
 
 fitCopula.itau <- function(copula, data, estimate.variance=TRUE) {
   q <- length(copula@parameters)
@@ -143,26 +135,25 @@ fitCopula.itau <- function(copula, data, estimate.variance=TRUE) {
   tau <- cor(data, method="kendall")
   itau <- fitKendall(copula, tau)
   itau <- itau[lower.tri(itau)]
-  if ("ellipCopula" %in% is(copula) && copula@dispstr == "ar1") { ## special treatment
-    estimate <- exp(lm(log(itau) ~ X - 1)$coef)
-  } else  estimate <- lm(itau ~ X - 1)$coef
+  estimate <-
+      if (is(copula, "ellipCopula") && copula@dispstr == "ar1") ## special treatment
+          exp(coef(lm(log(itau) ~ X - 1))) else coef(lm(itau ~ X - 1))
   attributes(estimate) <- NULL ## strip attributes
   copula@parameters[1:q] <- estimate
-  var.est <- if (estimate.variance) varKendall(copula, data) / nrow(data) else matrix(NA, q, q)
-  ans <- new("fitCopula",
-             estimate = estimate,
-             var.est = var.est,
-             method = "inversion of Kendall's tau",
-             loglik = as.numeric(NA), ## otherwise, class "fitCopula" complains.
-             convergence = as.integer(NA),
-             nsample = nrow(data),
-             copula = copula)
-  ans 
+  var.est <- if (estimate.variance)
+      varKendall(copula, data) / nrow(data) else matrix(NA, q, q)
+  new("fitCopula",
+      estimate = estimate,
+      var.est = var.est,
+      method = "inversion of Kendall's tau",
+      loglik = NA_real_,
+      convergence = NA_integer_,
+      nsample = nrow(data),
+      copula = copula)
 }
 
-###############################################################
-## fitCopula using inversion of Spearman's rho
-###############################################################
+
+## fitCopula using inversion of Spearman's rho #################################
 
 fitCopula.irho <- function(copula, data, estimate.variance=TRUE) {
   q <- length(copula@parameters)
@@ -170,26 +161,23 @@ fitCopula.irho <- function(copula, data, estimate.variance=TRUE) {
   rho <- cor(data, method="spearman")
   irho <- fitSpearman(copula, rho)
   irho <- irho[lower.tri(irho)]
-  if ("ellipCopula" %in% is(copula) && copula@dispstr == "ar1") { ## special treatment
-    estimate <- exp(lm(log(irho) ~ X - 1)$coef)
-  } else  estimate <- lm(irho ~ X - 1)$coef
+  estimate <- if (is(copula, "ellipCopula") && copula@dispstr == "ar1") ## special treatment
+      exp(coef(lm(log(irho) ~ X - 1))) else coef(lm(irho ~ X - 1))
   attributes(estimate) <- NULL ## strip attributes
   copula@parameters[1:q] <- estimate
   var.est <- if (estimate.variance) varSpearman(copula, data)/nrow(data) else matrix(NA, q, q)
-  ans <- new("fitCopula",
-             estimate = estimate,
-             var.est = var.est,
-             method = "inversion of Spearman's rho",
-             loglik = as.numeric(NA),
-             convergence = as.integer(NA),
-             nsample = nrow(data),
-             copula = copula)
-  ans
+  new("fitCopula",
+      estimate = estimate,
+      var.est = var.est,
+      method = "inversion of Spearman's rho",
+      loglik = as.numeric(NA),
+      convergence = as.integer(NA),
+      nsample = nrow(data),
+      copula = copula)
 }
 
-###############################################################
-## fitCopula using maximum pseudo-likelihood
-###############################################################
+
+## fitCopula using maximum pseudo-likelihood ###################################
 
 chkParamBounds <- function(copula) {
   param <- copula@parameters
@@ -203,7 +191,7 @@ chkParamBounds <- function(copula) {
 loglikCopula <- function(param, x, copula, hideWarnings=FALSE) {
   slot(copula, "parameters") <- param
   if (!chkParamBounds(copula)) return(NaN)
-  
+
   ## messageOut may be used for debugging
   if (hideWarnings) {
     messageOut <- textConnection("fitMessages", open="w", local=TRUE)
@@ -211,7 +199,7 @@ loglikCopula <- function(param, x, copula, hideWarnings=FALSE) {
     options(warn = -1) ## ignore warnings; can be undesirable!
   }
   loglik <- try(sum(log(dcopula(copula, x))))
-  
+
   if (hideWarnings) {
     options(warn = 0)
     sink(type="message"); sink(); close(messageOut)
@@ -227,9 +215,9 @@ fitCopula.ml <- function(data, copula, start=NULL,
                          estimate.variance=TRUE,
                          hideWarnings=FALSE) {
   if (copula@dimension != ncol(data))
-    stop("The dimention of the data and copula do not match.\n")
-  
-  if (is.null(start)) start <- fitCopula.itau(copula, data, FALSE)@estimate  
+    stop("The dimension of the data and copula do not match.\n")
+
+  if (is.null(start)) start <- fitCopula.itau(copula, data, FALSE)@estimate
   if (length(copula@parameters) != length(start))
     stop("The length of start and copula parameters do not match.\n")
 
@@ -240,8 +228,8 @@ fitCopula.ml <- function(data, copula, start=NULL,
   if (!is.null(optim.control[[1]])) control <- c(control, optim.control)
   q <- length(copula@parameters)
   eps <- .Machine$double.eps ^ 0.5
-  if (is.null(lower)) lower <- ifelse(method %in% c("Brent","L-BFGS-B"), copula@param.lowbnd + eps, -Inf)
-  if (is.null(upper)) upper <- ifelse(method %in% c("Brent","L-BFGS-B"), copula@param.upbnd - eps, Inf)
+  if (is.null(lower)) lower <- if(method %in% c("Brent","L-BFGS-B")) copula@param.lowbnd + eps else -Inf
+  if (is.null(upper)) upper <- if(method %in% c("Brent","L-BFGS-B")) copula@param.upbnd  - eps else Inf
 ##  if (p >= 2) {
   if (TRUE) {
     fit <- optim(start, loglikCopula,
@@ -273,43 +261,41 @@ fitCopula.ml <- function(data, copula, start=NULL,
     if (inherits(var.est, "try-error"))
       warning("Hessian matrix not invertible")
   } else var.est <- matrix(NA, q, q)
-  
-  ans <- new("fitCopula",
-             estimate = fit$par,
-             var.est = var.est,
-             method = "maximum likelihood",
-             loglik = loglik,
-             convergence = as.integer(convergence),
-             nsample = nrow(data),
-             copula = copula)
-  ans
+
+  new("fitCopula",
+      estimate = fit$par,
+      var.est = var.est,
+      method = "maximum likelihood",
+      loglik = loglik,
+      convergence = as.integer(convergence),
+      optimOpts = list(method = method, control = control),
+      nsample = nrow(data),
+      copula = copula)
 }
 
-#######################################################################
-## functions used in estimation and variance computation
-######################################################################
+
+## functions used in estimation and variance computation #######################
 
 ## taken from QRMlib and modified
 ## credit to Alexander McNeil and Scott Ulman
 makePosDef <- function (mat, delta = 0.001) {
   decomp <- eigen(mat)
   Lambda <- decomp$values
-  if (any(Lambda < 0) == TRUE) {
+  if (any(Lambda < 0)) {
     warning("Estimate is not positive-definite. Correction applied.")
     Lambda[Lambda < 0] <- delta
     Gamma <- decomp$vectors
     newmat <- Gamma %*% diag(Lambda) %*% t(Gamma)
     D <- 1/sqrt(diag(newmat))
-    return(diag(D) %*% newmat %*% diag(D))
+    diag(D) %*% newmat %*% diag(D)
   }
   else
-    return(mat)
+    mat
 }
 
-#########################################################
 ## rho given as a square matrix
 fitSpearman <- function(cop,rho)  {
-  p <- ncol(rho)
+  stopifnot(is.numeric(p <- ncol(rho)), p == nrow(rho))
   sigma <- matrix(1,p,p)
   for (j in 1:(p-1))
     for (i in (j+1):p)
@@ -317,18 +303,17 @@ fitSpearman <- function(cop,rho)  {
         sigma[i,j] <- calibSpearmansRho(cop,rho[i,j])
         sigma[j,i] <- sigma[i,j]
       }
-  
+
   ## make positive definite if necessary
-  if ("ellipCopula" %in% is(cop))
+  if (is(cop, "ellipCopula"))
     makePosDef(sigma, delta=0.001)
   else
     sigma
 }
 
-#########################################################
 ## tau given as a square matrix
 fitKendall <- function(cop,tau) {
-  p <- ncol(tau)
+  stopifnot(is.numeric(p <- ncol(tau)), p == nrow(tau))
   sigma <- matrix(1,p,p)
   for (j in 1:(p-1))
     for (i in (j+1):p)
@@ -336,31 +321,29 @@ fitKendall <- function(cop,tau) {
         sigma[i,j] <- calibKendallsTau(cop,tau[i,j])
         sigma[j,i] <- sigma[i,j]
       }
-  
+
   ## make positive definite if necessary
-  if ("ellipCopula" %in% is(cop))
+  if (is(cop, "ellipCopula"))
     makePosDef(sigma, delta=0.001)
   else
     sigma
 }
 
 
-###############################################################
-## variance/covariance of the pseudo-likelihood estimator
-###############################################################
+## variance/covariance of the pseudo-likelihood estimator ######################
 
 influ.terms <- function(u, influ, q)
 {
   p <- ncol(u)
   n <- nrow(u)
-  
+
   o <- ob <- matrix(0,n,p)
   for (i in 1:p)
     {
       o[,i] <- order(u[,i], decreasing=TRUE)
       ob[,i] <- rank(u[,i])
     }
-  
+
   out <- matrix(0,n,q)
   for (i in 1:p)
       out <- out + rbind(rep(0,q),apply(influ[[i]][o[,i],,drop=FALSE],2,cumsum))[n + 1 - ob[,i],,drop=FALSE] / n
@@ -377,12 +360,12 @@ varPL <- function(cop,u)
       q <- length(cop@parameters)
       return(matrix(NA, q, q))
     }
-    
+
     p <- cop@dimension
     n <- nrow(u)
-    
+
     ## influence: second part
-    ## integrals computed from the original pseudo-obs u by Monte Carlo 
+    ## integrals computed from the original pseudo-obs u by Monte Carlo
     dcop <- dcopwrap(cop,u) ## wrapper
     influ0 <- derPdfWrtParams(cop,u)/dcop
     derArg <- derPdfWrtArgs(cop,u)/dcop
@@ -392,14 +375,13 @@ varPL <- function(cop,u)
         influ[[i]] <- influ0 * derArg[,i]
 
     q <- length(cop@parameters)
-    inve <- solve(var(influ0)) 
-    
+    inve <- solve(var(influ0))
+
     return(inve %*% var(influ0 - influ.terms(u,influ,q)) %*% inve)
   }
 
-#################################################################################
-## variance of the estimator based on Kendall's tau
-#################################################################################
+
+## variance of the estimator based on Kendall's tau ############################
 
 ## cop is the FITTED copula
 ## u are the available pseudo-observations
@@ -410,8 +392,8 @@ getL <- function(copula) {
 
   dgidx <- outer(1:p, 1:p, "-")
   dgidx <- dgidx[lower.tri(dgidx)]
-  
-  if (!("ellipCopula" %in% is(copula))) {
+
+  if (!is(copula, "ellipCopula")) {
     matrix(1/pp, nrow=pp, ncol=1)
   } else if (copula@dispstr == "ex") {
     matrix(1/pp, nrow=pp, ncol=1)
@@ -436,8 +418,8 @@ getXmat <- function(copula) {
 
   dgidx <- outer(1:p, 1:p, "-")
   dgidx <- dgidx[lower.tri(dgidx)]
-  
-  if (!("ellipCopula" %in% is(copula))) { ## one-parameter non-elliptical copula
+
+  if (!is(copula, "ellipCopula")) { ## one-parameter non-elliptical copula
     matrix(1, nrow=pp, ncol=1)
   } else if (copula@dispstr == "ex") {
     matrix(1, nrow=pp, ncol=1)
@@ -460,22 +442,21 @@ varInfluAr1 <- function(cop, v, L, der) {
   p <- cop@dimension
   pp <- p * (p - 1) / 2
   n <- nrow(v)
-  
+
   ## estimate log(r) first, then log(theta), and then exponetiate back
   ## r is the lower.tri of sigma
   sigma <- getSigma(cop) # assuming cop is the fitted copula
   ## influ for log(r)
   r <- sigma[lower.tri(sigma)]
   der <- if (der == "tau") tauDerFun(cop)(r) else rhoDerFun(cop)(r)
-  D <- diag(pp)
-  diag(D) <-  1 / r / der
-  v <- v %*% D 
+  D <- diag(x = 1 / r / der, pp)
+  v <- v %*% D
   ## influ for log(theta)
   v <- v %*% L
-  
+
   ## influ for theta
   theta <- cop@parameters[1]
-  v %*% theta  
+  v %*% theta
 }
 
 varKendall <- function(cop,u) {
@@ -495,7 +476,7 @@ varKendall <- function(cop,u) {
   for (j in 1:(p-1)) {
     for (i in (j+1):p) {
       for (k in 1:n) ## can this be vectorized?
-        ec[k] <- sum(u[,i] <= u[k,i] & u[,j] <= u[k,j])/n 
+        ec[k] <- sum(u[,i] <= u[k,i] & u[,j] <= u[k,j])/n
       v[,l] <- 2 * ec - u[,i] - u[,j]
       l <- l + 1
     }
@@ -505,15 +486,14 @@ varKendall <- function(cop,u) {
   L <- t(solve(crossprod(X), t(X)))
   ## Caution: diag(0.5) is not a 1x1 matrix of 0.5!!! check it out.
   D <- if (length(cop@parameters) == 1) 1 / tauDer(cop) else diag(1 / tauDer(cop))
-  if ("ellipCopula" %in% is(cop) && cop@dispstr == "ar1") { ## special treatment
+  if (is(cop, "ellipCopula") && cop@dispstr == "ar1") { ## special treatment
     v <- varInfluAr1(cop, v, L, "tau")
     return(16 * var(v))
   } else return(16 * var(v %*% L %*% D))
 }
 
-##############################################################################
-## variance of the estimator based on Spearman's rho
-##############################################################################
+
+## variance of the estimator based on Spearman's rho ###########################
 
 ## cop is the FITTED copula
 ## u are the available pseudo-observations
@@ -525,28 +505,32 @@ varSpearman <- function(cop,u)  {
     return(matrix(NA, q, q))
   }
 
-  p <- cop@dimension
-  n <- nrow(u) 
+  stopifnot((p <- cop@dimension) >= 2)
+  n <- nrow(u)
   v <- matrix(0,n,p*(p-1)/2)
-    
+
   ord <- apply(u, 2, order, decreasing=TRUE)
   ordb <- apply(u, 2, rank)
 
   l <- 1
   for (j in 1:(p-1)) {
-    for (i in (j+1):p)  { 
-      v[,l] <- u[,i] * u[,j] + c(0, cumsum(u[ord[,i], j]))[n + 1 - ordb[,i]]/ n + c(0, cumsum(u[ord[,j], i]))[n + 1 - ordb[,j]] / n
+    for (i in (j+1):p)  {
+      v[,l] <- u[,i] * u[,j] +
+          c(0, cumsum(u[ord[,i], j]))[n + 1 - ordb[,i]] / n +
+          c(0, cumsum(u[ord[,j], i]))[n + 1 - ordb[,j]] / n
       l <- l + 1
     }
   }
   ## L <- getL(cop)
   X <- getXmat(cop)
   L <- t(solve(crossprod(X), t(X)))
-  ## Caution: diag(0.5) is not a 1x1 matrix of 0.5!!! check it out.
-  D <- if (length(cop@parameters) == 1) 1 / rhoDer(cop) else diag(1 / rhoDer(cop))
-  if ("ellipCopula" %in% is(cop) && cop@dispstr == "ar1") {
-    v <- varInfluAr1(cop, v, L, "rho")
-    return(144 * var(v))
-  } else return(144 * var(v %*% L %*% D))
+  if (is(cop, "ellipCopula") && cop@dispstr == "ar1") {
+      v <- varInfluAr1(cop, v, L, "rho")
+      144 * var(v)
+  } else {
+      ## Caution: diag(0.5) is not a 1x1 matrix of 0.5!!! check it out.
+      D <- if (length(cop@parameters) == 1) 1 / rhoDer(cop) else diag(1 / rhoDer(cop))
+      144 * var(v %*% L %*% D)
+  }
 }
 
