@@ -1,33 +1,30 @@
-/*#################################################################################
-##
-##   R package Copula by Jun Yan and Ivan Kojadinovic Copyright (C) 2008, 2009
-##
-##   This file is part of the R package copula.
-##
-##   The R package copula is free software: you can redistribute it and/or modify
-##   it under the terms of the GNU General Public License as published by
-##   the Free Software Foundation, either version 3 of the License, or
-##   (at your option) any later version.
-##
-##   The R package copula is distributed in the hope that it will be useful,
-##   but WITHOUT ANY WARRANTY; without even the implied warranty of
-##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-##   GNU General Public License for more details.
-##
-##   You should have received a copy of the GNU General Public License
-##   along with the R package copula. If not, see <http://www.gnu.org/licenses/>.
-##
-#################################################################################*/
+/*
+  Copyright (C) 2012 Marius Hofert, Ivan Kojadinovic, Martin Maechler, and Jun Yan
+
+  This program is free software; you can redistribute it and/or modify it under
+  the terms of the GNU General Public License as published by the Free Software
+  Foundation; either version 3 of the License, or (at your option) any later
+  version.
+
+  This program is distributed in the hope that it will be useful, but WITHOUT
+  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+  details.
+
+  You should have received a copy of the GNU General Public License along with
+  this program; if not, see <http://www.gnu.org/licenses/>.
+*/
 
 
 #include <R.h>
 #include <Rmath.h>
-#include "set.utils.h"
+
+#include "copula.h"
 
 /*****************************************************************************
 
-  Verifiy that the pdf of the FGM is positive at each vertex of [0,1]^p
-  Based on the polynominal used in the FGM copula 
+  Verify that the pdf of the FGM is positive at each vertex of [0,1]^p
+  Based on the polynomial used in the FGM copula
 
 *****************************************************************************/
 
@@ -37,7 +34,7 @@ void validity_fgm(int *p, double *alpha, int *valid)
   double prod, val;
   double *alpha_bin = Calloc(1<<*p, double);
   int *subset = Calloc(1<<*p, int);
-  
+
   k_power_set(p, p, subset);
   natural2binary(p, alpha, subset, alpha_bin);
 
@@ -46,9 +43,9 @@ void validity_fgm(int *p, double *alpha, int *valid)
     {
       val = 1.0;
       for (i=1;i<(1<<*p);i++)
-	if (card(i) > 1) 
+	if (card(i) > 1)
 	  {
-	    prod = alpha_bin[i]; 
+	    prod = alpha_bin[i];
 	    for (j=0;j<*p;j++)
 	      if ((1<<j) & i)
 		prod *= 1.0 - 2.0 * (((1<<j) & k) == (1<<j));
@@ -68,19 +65,20 @@ void validity_fgm(int *p, double *alpha, int *valid)
 
 /*****************************************************************************
 
-  Function to compute the conditional FGM copula 
-  Based on the polynominal used in the FGM copula 
-  alpha: params in binary order
+  Function to compute the conditional FGM copula
+  Based on the polynomial used in the FGM copula
+  alpha: para Ms in binary order
 
 *****************************************************************************/
 
+static
 double B(double *u, int p, int S, double *alpha)
 {
   int i, j;
   double prod, b = 1.0;
 
   for (i=1;i<(1<<p);i++)
-    if (card(i) > 1 && ((S & i) == i)) 
+    if (card(i) > 1 && ((S & i) == i))
       {
 	prod = alpha[i];
 	for (j=0;j<p;j++)
@@ -94,20 +92,21 @@ double B(double *u, int p, int S, double *alpha)
 /*****************************************************************************
 
   Function to compute the conditional FGM copula
-  Based on the polynominal used in the FGM copula 
+  Based on the polynomial used in the FGM copula
   alpha: params in binary order
 
 *****************************************************************************/
 
+static
 double A(double *u, int p, int S, int m, double *alpha)
 {
   int i, j;
   double prod, a = 0.0;
 
   for (i=1;i<(1<<p);i++)
-    if ((S & i) == i) 
+    if ((S & i) == i)
       {
-	prod = alpha[i + (1<<m)]; 
+	prod = alpha[i + (1<<m)];
 	for (j=0;j<p;j++)
 	  if ((1<<j) & S)
 	      prod *= 1.0 - 2.0 * u[j];
@@ -152,11 +151,12 @@ void rfgm(int *p, double *alpha, int *n, double *x)
 	  if (fabs(a) < 1e-16)
 	    x[i * (*p) + j] = unif_rand();
 	  else
-	    x[i * (*p) + j] = (a + b - sqrt((a + b) * (a + b) - 4.0 * a * b * unif_rand())) / (2.0 * a) ;
+	    x[i * (*p) + j] =
+		(a+b - sqrt((a+b)*(a+b) - 4.*a*b * unif_rand())) / (2.*a) ;
 	  S += (1<<j);
 	}
     }
-  
+
   PutRNGstate();
 
   Free(alpha_bin);
